% Correction application Chapitre 5
% Macroconomie quantitative (2014, d. Ellipses) 
% par Christophe Cahn

% Ce fichier permet de reproduire les lements contenu
% dans le fichier App_5_recursive.pdf disponible sur le site
% www.christophecahn.fr/macroquant

% Filename : App_5_recursive.m
% Application : Matlab/Octave
% Date : 24-04-2014
% Christophe Cahn (c) 2014

addpath('../MMMtbx');

clear all; clc; close all;
 
% 1.b
gam = 2;
bet = 0.95;
y   = [1.2 0.8];
pii = [0.6 0.4; 0.6 0.4];
ny  = length(y);
nw  = 500;
w_m = 2;
w   = linspace(0,w_m,nw)';

% 1.c
C = zeros(nw,nw,ny);
for kk = 1:ny
	C(:,:,kk) = repmat(w',nw,1) + y(kk)*ones(nw) - repmat(w,1,nw);
end
C(C<0)=0;
U = C.^(1-gam)./(1-gam);	

% 1.d
V = zeros(nw,ny) ;
g = zeros(nw,ny) ;

Vc = V*pii' ;
Vc = reshape(Vc,nw,1,ny) ;
Vc = repmat(Vc,[1 nw 1]) ;	

% 1.e
[TV,Tg] = max(U + bet*Vc);
TV = reshape(TV,nw,ny);
Tg = reshape(Tg,nw,ny);	

% 1.f
% initialisation pour l'exemple
ii = 1;

dV = max(max(abs(TV-V))) ;
eV = (dV <= 1.0e-06);
ei = (ii >= 100);
eg = isequal(g,Tg);
	
% 1.g
V = TV ;
g = Tg ;

% 1.h
% La boucle s'ecrit alors
V = zeros(nw,ny) ;
g = zeros(nw,ny) ;
eV = 0;
ei = 0;
eg = 0;
ii = 0;

while ~eV && ~ei && ~eg
	% calcul de la valeur de continuation
	Vc = V*pii' ;
	Vc = reshape(Vc,nw,1,ny) ;
	Vc = repmat(Vc,[1 nw 1]) ;	
	
	% contraction
	[TV,Tg] = max(U + bet*Vc);
	TV = reshape(TV,nw,ny);
	Tg = reshape(Tg,nw,ny);	
	
	% calcul des criteres d'arrt
	dV = max(max(abs(TV-V))) ;
	eV = (dV <= 1.0e-06);
	ei = (ii >= 100);
	eg = isequal(g,Tg);
	
	% mise a jour
	V = TV ;
	g = Tg ;
	ii = ii + 1;
end	

% 1.i
if eg
	disp('Regle de decision invariante.');
elseif ei
	disp('Maximum d''iterations atteint.');
else
	disp('Convergence obtenue.');
end	

% 1.j
W = w(g) ;	
c = repmat(w,1,2) + repmat(y,nw,1) - W ;

% 1.k
figure;
plot(w,W(:,1),'b',w,W(:,2),'r',w,w,':');
legend('dotation haute','dotation basse');
title('Choix optimal de richesse en fin de periode');
print('./App_5_fig_W.pdf','-dpdf')	

figure;
plot(w,c(:,1),'b',w,c(:,2),'r');
legend('dotation haute','dotation basse');	
title('Choix optimal de consommation');
print('./App_5_fig_c.pdf','-dpdf')	

% 1.l
% On recommence

gam = .5 ;
for kk = 1:ny
	C(:,:,kk) = repmat(w',nw,1) + y(kk)*ones(nw) - repmat(w,1,nw);
end
C(C<0)=0;
U = C.^(1-gam)./(1-gam);

V = zeros(nw,ny) ;
g = zeros(nw,ny) ;
eV = 0;
ei = 0;
eg = 0;
ii = 0;

while ~eV && ~ei && ~eg
	% calcul de la valeur de continuation
	Vc = V*pii' ;
	Vc = reshape(Vc,nw,1,ny) ;
	Vc = repmat(Vc,[1 nw 1]) ;	
	
	% contraction
	[TV,Tg] = max(U + bet*Vc);
	TV = reshape(TV,nw,ny);
	Tg = reshape(Tg,nw,ny);	
	
	% calcul des criteres d'arrt
	dV = max(max(abs(TV-V))) ;
	eV = (dV <= 1.0e-06);
	ei = (ii >= 100);
	eg = isequal(g,Tg);
	
	% mise a jour
	V = TV ;
	g = Tg ;
	ii = ii + 1;
end

W2 = w(g) ;	
c2 = repmat(w,1,2) + repmat(y,nw,1) - W2 ;

figure;
plot(w,W(:,1),'b',w,W(:,2),'r',w,W2(:,1),'--b',w,W2(:,2),'--r',w,w,':k');
legend('dotation haute \sigma=2','dotation basse \sigma=2',...
		'dotation haute \sigma=.5','dotation basse \sigma=.5');
title('Choix optimal de richesse en fin de periode');
print('../pdf/App_5_fig_W2.pdf','-dpdf')	

figure;
plot(w,c(:,1),'b',w,c(:,2),'r',w,c2(:,1),'--b',w,c2(:,2),'--r');
legend('dotation haute \sigma=2','dotation basse \sigma=2',...
		'dotation haute \sigma=.5','dotation basse \sigma=.5');
title('Choix optimal de consommation');
print('../pdf/App_5_fig_c2.pdf','-dpdf')	

%% Le modele de croissance neoclassique
fprintf('Le modele de croissance neoclassique\n')
fprintf('\n')
% parametres et grille
bet  = 0.95;
alp  = 0.36;
A    = 1;
delt = 1;
kbar = 2*(alp*A/(1/bet-1+delt))^(1/(1-alp)) ;
N	 = 100;
kmin = 0.02;
K 	 = linspace(kmin,1.1*kbar,N)';
u	 = @(c) log(c);
Vbar = u(A*kbar^alp-delt*kbar)/(1-bet);

% initialisation
V  = ones(N,1)*Vbar;
TV = zeros(N,1);
g  = ones(N,1)*kbar;
Tg = zeros(N,1);
n  = 0;
eV = 0;
eg = 0;
en = 0;

% iteration

% figure()
while ~eV && ~en && ~eg
fV = @(k) interp1(K,V,k,'linear');
for ii = 1:N
	TfV = @(k) u(max(A*K(ii).^alp+(1-delt)*K(ii)-k,1e-15)) + bet*fV(k);

	[kmax,Vmax] = goldsecrule(TfV,K([1 end]));
	
	Tg(ii) = kmax ;
	TV(ii) = Vmax ;
end

% criteres d'arrt

dV = max(max(abs(TV-V)))/(1+max(max(abs(TV)))) ;
eV = (dV <= 1.0e-06) ;
dg = max(max(abs(Tg-g)))/(1+max(max(abs(Tg)))) ;
eg = (dg <= 1.0e-06);
en = (n >= 100);
% pause
fprintf('dg= %f , dV= %f on iter %d \n',dg,dV,n);
% mise a jour
V = TV ;
g = Tg ;
n = n + 1 ;

end

if eg
	disp('Invariance de la regle de decision.');
elseif en
	disp('Nombre d''iteration maximum atteint.');
else
	disp('Convergence de la fonction valeur.');
end	


C = A*K.^alp+(1-delt)*K-g;
C_exact = (1-alp*bet)*A*K.^alp;
g_exact = alp*bet*A*K.^alp;

figure;
plot(K,C,'b',K,C_exact,'r');
legend('Consommation - VFI','Consommation - exact')
print('./App_5_fig_cons.pdf','-dpdf')	

figure;
plot(K,g,'b',K,g_exact,'r',K,K,':');
legend('Capital - VFI','Capital - exact')	
print('./App_5_fig_cap.pdf','-dpdf')	

